-- package Ada95_Parser for SCATC DSK
--   Modified from the original by Richard Conn

------------------------------------------------------------------------------

-- Single Character Tokens

%token '&' ''' '(' ')' '*' '+' ',' '-' '.' '/' ':' ';'
%token '<' '=' '>' '|'


-- Special Character or Two Character Tokens

%token ARROW DOUBLE_DOT DOUBLE_STAR ASSIGNMENT INEQUALITY 
%token GREATER_THAN_OR_EQUAL LESS_THAN_OR_EQUAL
%token LEFT_LABEL_BRACKET RIGHT_LABEL_BRACKET 
%token BOX


-- Reserved Words

%token ABORT_TOKEN ABS_TOKEN ABSTRACT_TOKEN ACCEPT_TOKEN  ACCESS_TOKEN
%token ALIASED_TOKEN ALL_TOKEN  AND_TOKEN ARRAY_TOKEN AT_TOKEN

%token BEGIN_TOKEN  BODY_TOKEN BOGUS_TOKEN

%token CASE_TOKEN  CONSTANT_TOKEN

%token DECLARE_TOKEN DELAY_TOKEN DELTA_TOKEN DIGITS_TOKEN DO_TOKEN 

%token ELSE_TOKEN ELSIF_TOKEN END_TOKEN  ENTRY_TOKEN EXCEPTION_TOKEN 
%token EXIT_TOKEN 

%token FOR_TOKEN FUNCTION_TOKEN 

%token GENERIC_TOKEN  GOTO_TOKEN

%token IF_TOKEN IN_TOKEN  IS_TOKEN

%token LIMITED_TOKEN LOOP_TOKEN 

%token MOD_TOKEN

%token NEW_TOKEN NOT_TOKEN  NULL_TOKEN

%token OF_TOKEN OR_TOKEN OTHERS_TOKEN OUT_TOKEN  

%token PACKAGE_TOKEN PRAGMA_TOKEN  PRIVATE_TOKEN PROCEDURE_TOKEN  
%token PROTECTED_TOKEN

%token RAISE_TOKEN  RANGE_TOKEN RECORD_TOKEN REM_TOKEN RENAMES_TOKEN 
%token REQUEUE_TOKEN RETURN_TOKEN REVERSE_TOKEN

%token SELECT_TOKEN  SEPARATE_TOKEN SUBTYPE_TOKEN

%token TAGGED_TOKEN TASK_TOKEN TERMINATE_TOKEN THEN_TOKEN TYPE_TOKEN 

%token UNTIL_TOKEN USE_TOKEN

%token WHEN_TOKEN WHILE_TOKEN WITH_TOKEN

%token XOR_TOKEN


-- Additional Tokens

%token BASE
%token BASED_LITERAL

%token DECIMAL_LITERAL
%token DIGIT

%token EXTENDED_DIGIT
%token EXPONENT 

%token IDENTIFIER 
-- %token INTEGER 

%token CHAR_LITERAL STRING_LITERAL 

-- %token ERROR1 ERROR2  ERROR3  ERROR4  ERROR5  ERROR6  ERROR7  ERROR8
-- %token ERROR9 ERROR10 ERROR11 ERROR12 ERROR13 ERROR14 ERROR15

%start compilation

{ 
  Maximum_String_Length : constant := 200;
  type STRING_BUFFER is record
    Text        : STRING(1..Maximum_String_Length);
    Last        : NATURAL := 0;
  end record;

  type YYSTYPE is record
    Name        : STRING_BUFFER;
    Line        : NATURAL := 0;
  end record;
}
 
%%


goal_symbol : compilation
	;

NUMERIC_LIT : DECIMAL_LITERAL
            | BASED_LITERAL
            ;

CHAR_LIT : CHAR_LITERAL
            ;

GT_GT : RIGHT_LABEL_BRACKET
      ;

LT_LT : LEFT_LABEL_BRACKET
      ;

GE : GREATER_THAN_OR_EQUAL
   ;

LT_EQ : LESS_THAN_OR_EQUAL
      ;

NE : INEQUALITY
   ;

CHAR_STRING : STRING_LITERAL
   ;

TIC : '''
    ;

pragma  : PRAGMA_TOKEN identifier ';'
	| PRAGMA_TOKEN simple_name '(' pragma_arg_s ')' ';'
	;

pragma_arg_s : pragma_arg
	| pragma_arg_s ',' pragma_arg
	;

pragma_arg : expression
	| simple_name ARROW expression
	;

pragma_s :
	| pragma_s pragma
	;

decl    : object_decl
	| number_decl
	| type_decl
	| subtype_decl
	| subprog_decl
	| pkg_decl
	| task_decl
	| prot_decl
	| exception_decl
	| rename_decl
	| generic_decl
	| body_stub
	| error ';'
	;

object_decl : def_id_s ':' object_qualifier_opt object_subtype_def init_opt ';'
	;

def_id_s : def_id
	| def_id_s ',' def_id
	;

def_id  : identifier
	;

object_qualifier_opt :
	| ALIASED_TOKEN
	| CONSTANT_TOKEN
	| ALIASED_TOKEN CONSTANT_TOKEN
	;

object_subtype_def : subtype_ind
	| array_type
	;

init_opt :
	| ASSIGNMENT expression
	;

number_decl : def_id_s ':' CONSTANT_TOKEN ASSIGNMENT expression ';'
	;

type_decl : TYPE_TOKEN identifier discrim_part_opt type_completion ';'
	;

discrim_part_opt :
	| discrim_part
	| '(' BOX ')'
	;

type_completion :
	| IS_TOKEN type_def
	;

type_def : enumeration_type 
	| integer_type
	| real_type
	| array_type
	| record_type
	| access_type
	| derived_type
	| private_type
	;

subtype_decl : SUBTYPE_TOKEN identifier IS_TOKEN subtype_ind ';'
	;

subtype_ind : name constraint
	| name
	;

constraint : range_constraint
	| decimal_digits_constraint
	;

decimal_digits_constraint : DIGITS_TOKEN expression range_constr_opt
	;

derived_type : NEW_TOKEN subtype_ind
	| NEW_TOKEN subtype_ind WITH_TOKEN PRIVATE_TOKEN
	| NEW_TOKEN subtype_ind WITH_TOKEN record_def
	| ABSTRACT_TOKEN NEW_TOKEN subtype_ind WITH_TOKEN PRIVATE_TOKEN
	| ABSTRACT_TOKEN NEW_TOKEN subtype_ind WITH_TOKEN record_def
	;

range_constraint : RANGE_TOKEN range
	;

range : simple_expression DOUBLE_DOT simple_expression
	| name TIC RANGE_TOKEN
	| name TIC RANGE_TOKEN '(' expression ')'
	;

enumeration_type : '(' enum_id_s ')'
        ;

enum_id_s : enum_id
	| enum_id_s ',' enum_id
	;

enum_id : identifier
	| char_lit
	;

integer_type : range_spec
	| MOD_TOKEN expression
	;
	

range_spec : range_constraint
	;

range_spec_opt :
	| range_spec
	;

real_type : float_type
	| fixed_type
	;

float_type : DIGITS_TOKEN expression range_spec_opt
	;

fixed_type : DELTA_TOKEN expression range_spec
	| DELTA_TOKEN expression DIGITS_TOKEN expression range_spec_opt
	;

array_type : unconstr_array_type
	| constr_array_type
	;

unconstr_array_type : ARRAY_TOKEN '(' index_s ')' OF_TOKEN component_subtype_def
	;

constr_array_type : ARRAY_TOKEN iter_index_constraint OF_TOKEN component_subtype_def
	;

component_subtype_def : aliased_opt subtype_ind
	;

aliased_opt : 
	| ALIASED_TOKEN
	;

index_s : index
	| index_s ',' index
	;

index : name RANGE_TOKEN BOX
	;

iter_index_constraint : '(' iter_discrete_range_s ')'
	;

iter_discrete_range_s : discrete_range
	| iter_discrete_range_s ',' discrete_range
	;

discrete_range : name range_constr_opt
	| range
	;

range_constr_opt :
	| range_constraint
	;

record_type : tagged_opt limited_opt record_def
	;

record_def : RECORD_TOKEN pragma_s comp_list END_TOKEN RECORD_TOKEN
	| NULL_TOKEN RECORD_TOKEN
	;

tagged_opt :
	| TAGGED_TOKEN
	| ABSTRACT_TOKEN TAGGED_TOKEN
	;

comp_list : comp_decl_s variant_part_opt
	| variant_part pragma_s
	| NULL_TOKEN ';' pragma_s
	;

comp_decl_s : comp_decl
	| comp_decl_s pragma_s comp_decl
	;

variant_part_opt : pragma_s
	| pragma_s variant_part pragma_s
	;

comp_decl : def_id_s ':' component_subtype_def init_opt ';'
	| error ';'
	;

discrim_part : '(' discrim_spec_s ')'
	;

discrim_spec_s : discrim_spec
	| discrim_spec_s ';' discrim_spec
	;

discrim_spec : def_id_s ':' access_opt mark init_opt
	| error
	;

access_opt :
	| ACCESS_TOKEN
	;

variant_part : CASE_TOKEN simple_name IS_TOKEN pragma_s variant_s END_TOKEN CASE_TOKEN ';'
	;

variant_s : variant
	| variant_s variant
	;

variant : WHEN_TOKEN choice_s ARROW pragma_s comp_list
	;

choice_s : choice
	| choice_s '|' choice
	;

choice : expression
	| discrete_with_range
	| OTHERS_TOKEN
	;

discrete_with_range : name range_constraint
	| range
	;

access_type : ACCESS_TOKEN subtype_ind
	| ACCESS_TOKEN CONSTANT_TOKEN subtype_ind
	| ACCESS_TOKEN ALL_TOKEN subtype_ind
	| ACCESS_TOKEN prot_opt PROCEDURE_TOKEN formal_part_opt
	| ACCESS_TOKEN prot_opt FUNCTION_TOKEN formal_part_opt RETURN_TOKEN mark
	;

prot_opt :
	| PROTECTED_TOKEN
	;

decl_part :
	| decl_item_or_body_s1
	;

decl_item_s : 
	| decl_item_s1
	;

decl_item_s1 : decl_item
	| decl_item_s1 decl_item
	;

decl_item : decl
	| use_clause
	| rep_spec
	| pragma
	;

decl_item_or_body_s1 : decl_item_or_body
	| decl_item_or_body_s1 decl_item_or_body
	;

decl_item_or_body : body
	| decl_item
	;

body : subprog_body
	| pkg_body
	| task_body
	| prot_body
	;

name : simple_name
	| indexed_comp
	| selected_comp
	| attribute
	| operator_symbol
	;

mark : simple_name
	| mark TIC attribute_id
	| mark '.' simple_name
	;

simple_name : identifier
	;

compound_name : simple_name
	| compound_name '.' simple_name
	;

c_name_list : compound_name
	 | c_name_list ',' compound_name
	;

used_char : char_lit
	;

operator_symbol : char_string
	;

indexed_comp : name '(' value_s ')'
	;

value_s : value
	| value_s ',' value
	;

value : expression
	| comp_assoc
	| discrete_with_range
	| error
	;

selected_comp : name '.' simple_name
	| name '.' used_char
	| name '.' operator_symbol
	| name '.' ALL_TOKEN
	;

attribute : name TIC attribute_id
	;

attribute_id : identifier
	| DIGITS_TOKEN
	| DELTA_TOKEN
	| ACCESS_TOKEN
	;

literal : numeric_lit
	| used_char
	| NULL_TOKEN
	;

aggregate : '(' comp_assoc ')'
	| '(' value_s_2 ')'
	| '(' expression WITH_TOKEN value_s ')'
	| '(' expression WITH_TOKEN NULL_TOKEN RECORD_TOKEN ')'
	| '(' NULL_TOKEN RECORD_TOKEN ')'
	;

value_s_2 : value ',' value
	| value_s_2 ',' value
	;

comp_assoc : choice_s ARROW expression
	;

expression : relation
	| expression logical relation
	| expression short_circuit relation
	;

logical : AND_TOKEN
	| OR_TOKEN
	| XOR_TOKEN
	;

short_circuit : AND_TOKEN THEN_TOKEN
	| OR_TOKEN ELSE_TOKEN
	;

relation : simple_expression
	| simple_expression relational simple_expression
	| simple_expression membership range
	| simple_expression membership name
	;

relational : '='
	| NE
	| '<'
	| LT_EQ
	| '>'
	| GE
	;

membership : IN_TOKEN
	| NOT_TOKEN IN_TOKEN
	;

simple_expression : unary term
	| term
	| simple_expression adding term
	;

unary   : '+'
	| '-'
	;

adding  : '+'
	| '-'
	| '&'
	;

term    : factor
	| term multiplying factor
	;

multiplying : '*'
	| '/'
	| MOD_TOKEN
	| REM_TOKEN
	;

factor : primary
	| NOT_TOKEN primary
	| ABS_TOKEN primary
	| primary EXPONENT primary
	;

primary : literal
	| name
	| allocator
	| qualified
	| parenthesized_primary
	;

parenthesized_primary : aggregate
	| '(' expression ')'
	;

qualified : name TIC parenthesized_primary
	;

allocator : NEW_TOKEN name
	| NEW_TOKEN qualified
	;

statement_s : statement
	| statement_s statement
	;

statement : unlabeled
	| label statement
	;

unlabeled : simple_stmt
	| compound_stmt
	| pragma
	;

simple_stmt : null_stmt
	| assign_stmt
	| exit_stmt
	| return_stmt
	| goto_stmt
	| procedure_call
	| delay_stmt
	| abort_stmt
	| raise_stmt
	| code_stmt
	| requeue_stmt
	| error ';'
	;

compound_stmt : if_stmt
	| case_stmt
	| loop_stmt
	| block
	| accept_stmt
	| select_stmt
	;

label : LT_LT identifier GT_GT
	;

null_stmt : NULL_TOKEN ';'
	;

assign_stmt : name ASSIGNMENT expression ';'
	;

if_stmt : IF_TOKEN cond_clause_s else_opt END_TOKEN IF_TOKEN ';'
	;

cond_clause_s : cond_clause
	| cond_clause_s ELSIF_TOKEN cond_clause
	;

cond_clause : cond_part statement_s
	;

cond_part : condition THEN_TOKEN
	;

condition : expression
	;

else_opt :
	| ELSE_TOKEN statement_s
	;

case_stmt : case_hdr pragma_s alternative_s END_TOKEN CASE_TOKEN ';'
	;

case_hdr : CASE_TOKEN expression IS_TOKEN
	;

alternative_s :
	| alternative_s alternative
	;

alternative : WHEN_TOKEN choice_s ARROW statement_s
	;

loop_stmt : label_opt iteration basic_loop id_opt ';'
	;

label_opt :
	| identifier ':'
	;

iteration :
	| WHILE_TOKEN condition
	| iter_part reverse_opt discrete_range
	;

iter_part : FOR_TOKEN identifier IN_TOKEN
	;

reverse_opt :
	| REVERSE_TOKEN
	;

basic_loop : LOOP_TOKEN statement_s END_TOKEN LOOP_TOKEN
	;

id_opt :
	| designator
	;

block : label_opt block_decl block_body END_TOKEN id_opt ';'
	;

block_decl :
	| DECLARE_TOKEN decl_part
	;

block_body : BEGIN_TOKEN handled_stmt_s
	;

handled_stmt_s : statement_s except_handler_part_opt 
	; 

except_handler_part_opt :
	| except_handler_part
	;

exit_stmt : EXIT_TOKEN name_opt when_opt ';'
	;

name_opt :
	| name
	;

when_opt :
	| WHEN_TOKEN condition
	;

return_stmt : RETURN_TOKEN ';'
	| RETURN_TOKEN expression ';'
	;

goto_stmt : GOTO_TOKEN name ';'
	;

subprog_decl : subprog_spec ';'
	| generic_subp_inst ';'
	| subprog_spec_is_push ABSTRACT_TOKEN ';'
	;

subprog_spec : PROCEDURE_TOKEN compound_name formal_part_opt
	| FUNCTION_TOKEN designator formal_part_opt RETURN_TOKEN name
	| FUNCTION_TOKEN designator --  for generic inst and generic rename
	;

designator : compound_name
	| char_string
	;

formal_part_opt : 
	| formal_part
	;

formal_part : '(' param_s ')'
	;

param_s : param
	| param_s ';' param
	;

param : def_id_s ':' mode mark init_opt
	| error
	;

mode :
	| IN_TOKEN
	| OUT_TOKEN
	| IN_TOKEN OUT_TOKEN
	| ACCESS_TOKEN
	;

subprog_spec_is_push : subprog_spec IS_TOKEN
	;

subprog_body : subprog_spec_is_push
	       decl_part block_body END_TOKEN id_opt ';'
	;

procedure_call : name ';'
	;

pkg_decl : pkg_spec ';'
	| generic_pkg_inst ';'
	;

pkg_spec : PACKAGE_TOKEN compound_name IS_TOKEN 
	     decl_item_s private_part END_TOKEN c_id_opt
       ;
private_part :
	| PRIVATE_TOKEN decl_item_s
	;

c_id_opt : 
	| compound_name
	;

pkg_body : PACKAGE_TOKEN BODY_TOKEN compound_name IS_TOKEN
	       decl_part body_opt END_TOKEN c_id_opt ';'
	;

body_opt :
	| block_body
	;

private_type : tagged_opt limited_opt PRIVATE_TOKEN
	;

limited_opt :
	| LIMITED_TOKEN
	;

use_clause : USE_TOKEN name_s ';'
	| USE_TOKEN TYPE_TOKEN name_s ';'
	;

name_s : name
	| name_s ',' name
	;

rename_decl : def_id_s ':' object_qualifier_opt subtype_ind renames ';'
	| def_id_s ':' EXCEPTION_TOKEN renames ';'
	| rename_unit
	;

rename_unit : PACKAGE_TOKEN compound_name renames ';'
	| subprog_spec renames ';'
	| generic_formal_part PACKAGE_TOKEN compound_name renames ';'
	| generic_formal_part subprog_spec renames ';'
	;

renames : RENAMES_TOKEN name
	;

task_decl : task_spec ';'
	;

task_spec : TASK_TOKEN simple_name task_def
	| TASK_TOKEN TYPE_TOKEN simple_name discrim_part_opt task_def
	;

task_def :
	| IS_TOKEN entry_decl_s rep_spec_s task_private_opt END_TOKEN id_opt
	;

task_private_opt :
	| PRIVATE_TOKEN entry_decl_s rep_spec_s
	;

task_body : TASK_TOKEN BODY_TOKEN simple_name IS_TOKEN
	       decl_part block_body END_TOKEN id_opt ';'
	;

prot_decl : prot_spec ';'
	;

prot_spec : PROTECTED_TOKEN identifier prot_def
	| PROTECTED_TOKEN TYPE_TOKEN simple_name discrim_part_opt prot_def
	;

prot_def : IS_TOKEN prot_op_decl_s prot_private_opt END_TOKEN id_opt
	;

prot_private_opt :
	| PRIVATE_TOKEN prot_elem_decl_s 
        ;
     
prot_op_decl_s : 
	| prot_op_decl_s prot_op_decl
	;

prot_op_decl : entry_decl
	| subprog_spec ';'
	| rep_spec
	| pragma
	;

prot_elem_decl_s : 
	| prot_elem_decl_s prot_elem_decl
	;

prot_elem_decl : prot_op_decl | comp_decl ;

prot_body : PROTECTED_TOKEN BODY_TOKEN simple_name IS_TOKEN
	       prot_op_body_s END_TOKEN id_opt ';'
	;

prot_op_body_s : pragma_s
	| prot_op_body_s prot_op_body pragma_s
	;

prot_op_body : entry_body
	| subprog_body
	| subprog_spec ';'
	;

entry_decl_s : pragma_s
	| entry_decl_s entry_decl pragma_s
	;

entry_decl : ENTRY_TOKEN identifier formal_part_opt ';'
	| ENTRY_TOKEN identifier '(' discrete_range ')' formal_part_opt ';'
	;

entry_body : ENTRY_TOKEN identifier formal_part_opt WHEN_TOKEN 
                condition entry_body_part
	| ENTRY_TOKEN identifier '(' iter_part discrete_range ')' 
		formal_part_opt WHEN_TOKEN condition entry_body_part
	;

entry_body_part : ';'
	| IS_TOKEN decl_part block_body END_TOKEN id_opt ';'
	;

rep_spec_s :
	| rep_spec_s rep_spec pragma_s
	;

entry_call : procedure_call
	;

accept_stmt : accept_hdr ';'
	| accept_hdr DO_TOKEN handled_stmt_s END_TOKEN id_opt ';'
	;

accept_hdr : ACCEPT_TOKEN entry_name formal_part_opt
	;

entry_name : simple_name
	| entry_name '(' expression ')'
	;

delay_stmt : DELAY_TOKEN expression ';'
	| DELAY_TOKEN UNTIL_TOKEN expression ';'
	;

select_stmt : select_wait
	| async_select
	| timed_entry_call
	| cond_entry_call
	;

select_wait : SELECT_TOKEN guarded_select_alt or_select else_opt
	      END_TOKEN SELECT_TOKEN ';'
	;

guarded_select_alt : select_alt
	| WHEN_TOKEN condition ARROW select_alt
	;

or_select :
	| or_select OR_TOKEN guarded_select_alt
	;

select_alt : accept_stmt stmts_opt
	| delay_stmt stmts_opt
	| TERMINATE_TOKEN ';'
	;

delay_or_entry_alt : delay_stmt stmts_opt
	| entry_call stmts_opt
        ;

async_select : SELECT_TOKEN delay_or_entry_alt
	       THEN_TOKEN ABORT_TOKEN statement_s
	       END_TOKEN SELECT_TOKEN ';'
	;

timed_entry_call : SELECT_TOKEN entry_call stmts_opt 
		   OR_TOKEN delay_stmt stmts_opt
	           END_TOKEN SELECT_TOKEN ';'
	;

cond_entry_call : SELECT_TOKEN entry_call stmts_opt 
		  ELSE_TOKEN statement_s
	          END_TOKEN SELECT_TOKEN ';'
	;

stmts_opt :
	| statement_s
	;

abort_stmt : ABORT_TOKEN name_s ';'
	;

compilation :
	| compilation comp_unit
	| pragma pragma_s
	;

comp_unit : context_spec private_opt unit pragma_s
	| private_opt unit pragma_s
	;

private_opt :
	| PRIVATE_TOKEN
	;

context_spec : with_clause use_clause_opt
	| context_spec with_clause use_clause_opt
	| context_spec pragma
	;

with_clause : WITH_TOKEN c_name_list ';'
	;

use_clause_opt :
	| use_clause_opt use_clause
	;

unit : pkg_decl
	| pkg_body
	| subprog_decl
	| subprog_body
	| subunit
	| generic_decl
	| rename_unit
	;

subunit : SEPARATE_TOKEN '(' compound_name ')'
	      subunit_body
	;

subunit_body : subprog_body
	| pkg_body
	| task_body
	| prot_body
	;

body_stub : TASK_TOKEN BODY_TOKEN simple_name IS_TOKEN SEPARATE_TOKEN ';'
	| PACKAGE_TOKEN BODY_TOKEN compound_name IS_TOKEN SEPARATE_TOKEN ';'
	| subprog_spec IS_TOKEN SEPARATE_TOKEN ';'
	| PROTECTED_TOKEN BODY_TOKEN simple_name IS_TOKEN SEPARATE_TOKEN ';'
	;

exception_decl : def_id_s ':' EXCEPTION_TOKEN ';'
	;

except_handler_part : EXCEPTION_TOKEN exception_handler
	| except_handler_part exception_handler
	;

exception_handler : WHEN_TOKEN except_choice_s ARROW statement_s
	| WHEN_TOKEN identifier ':' except_choice_s ARROW statement_s
	;

except_choice_s : except_choice
	| except_choice_s '|' except_choice
	;

except_choice : name
	| OTHERS_TOKEN
	;

raise_stmt : RAISE_TOKEN name_opt ';'
	;

requeue_stmt : REQUEUE_TOKEN name ';'
	| REQUEUE_TOKEN name WITH_TOKEN ABORT_TOKEN ';'
	;

generic_decl : generic_formal_part subprog_spec ';'
	| generic_formal_part pkg_spec ';'
	;

generic_formal_part : GENERIC_TOKEN
	| generic_formal_part generic_formal
	;

generic_formal : param ';'
	| TYPE_TOKEN simple_name generic_discrim_part_opt IS_TOKEN 
             generic_type_def ';'
	| WITH_TOKEN PROCEDURE_TOKEN simple_name 
	    formal_part_opt subp_default ';'
	| WITH_TOKEN FUNCTION_TOKEN designator 
	    formal_part_opt RETURN_TOKEN name subp_default ';'
	| WITH_TOKEN PACKAGE_TOKEN simple_name IS_TOKEN NEW_TOKEN 
            name '(' BOX ')' ';'
	| WITH_TOKEN PACKAGE_TOKEN simple_name IS_TOKEN NEW_TOKEN name ';'
	| use_clause
	;

generic_discrim_part_opt :
	| discrim_part
	| '(' BOX ')'
	;

subp_default :
	| IS_TOKEN name
	| IS_TOKEN BOX
	;

generic_type_def : '(' BOX ')'
	| RANGE_TOKEN BOX
	| MOD_TOKEN BOX
	| DELTA_TOKEN BOX
	| DELTA_TOKEN BOX DIGITS_TOKEN BOX
	| DIGITS_TOKEN BOX
	| array_type
	| access_type
	| private_type
	| generic_derived_type
	;

generic_derived_type : NEW_TOKEN subtype_ind
	| NEW_TOKEN subtype_ind WITH_TOKEN PRIVATE_TOKEN
	| ABSTRACT_TOKEN NEW_TOKEN subtype_ind WITH_TOKEN PRIVATE_TOKEN
	;

generic_subp_inst : subprog_spec IS_TOKEN generic_inst
	;

generic_pkg_inst : PACKAGE_TOKEN compound_name IS_TOKEN generic_inst
	;

generic_inst : NEW_TOKEN name
	;

rep_spec : attrib_def
	| record_type_spec
	| address_spec
	;

attrib_def : FOR_TOKEN mark USE_TOKEN expression ';'
	;

record_type_spec : FOR_TOKEN mark USE_TOKEN RECORD_TOKEN
                     align_opt comp_loc_s END_TOKEN RECORD_TOKEN ';'
	;

align_opt :
	| AT_TOKEN MOD_TOKEN expression ';'
	;

comp_loc_s :
	| comp_loc_s mark AT_TOKEN expression RANGE_TOKEN range ';'
	;

address_spec : FOR_TOKEN mark USE_TOKEN AT_TOKEN expression ';'
	;

code_stmt : qualified ';'
	;


%%

-- ***********************************************************
-- *                                                         *
-- * Ada95_Parser                                            * SPEC
-- *                                                         *
-- ***********************************************************
package A95P is

  procedure Run (File_Name : in STRING);
  --| PURPOSE:
  --| Cause the Parser to execute.  This is simply a call to YYParse.
  
  procedure Display_Results;
  --| PURPOSE:
  --| Display final results on all files parsed so far.

end A95P;

-- ***********************************************************
-- *                                                         *
-- * Ada95_Parser                                            * BODY
-- *                                                         *
-- ***********************************************************
with A95;               use A95;
with A95P_Goto;         use A95P_Goto;
with A95P_Shift_Reduce; use A95P_Shift_Reduce;
with A95P_Tokens;       use A95P_Tokens;
with Text_IO;           use Text_IO;
package body A95P is

------------------------------------------------------------------
-- Local types and objects
------------------------------------------------------------------

  Echo             : BOOLEAN := FALSE;
  Number_of_Errors : NATURAL := 0;

------------------------------------------------------------------
-- Local routines
------------------------------------------------------------------

  -- ***********************************************************
  -- *                                                         *
  -- * Ada95_Parser.Language_YY2_Lexical_Analyzer              *
  -- *                                                         *
  -- ***********************************************************
  package Language_YY2_Lexical_Analyzer renames A95;
  use Language_YY2_Lexical_Analyzer;
  -- This redefinition is required for the parser which is generated
  -- using the SCATC-customized version of ayacc.  It expects
  -- the tokens to come from a Language_YY2_Lexical_Analyzer, so we
  -- must give it one.

  -- ***********************************************************
  -- *                                                         *
  -- * Ada95_Parser.YYError                                    *
  -- *                                                         *
  -- ***********************************************************
  procedure YYError(S: in STRING := "Syntax Error") is
  --| PURPOSE:
  --| Output error message and increment Number_of_Errors
  begin -- YYError
    Number_of_Errors := Number_of_Errors + 1;
  end YYError;

------------------------------------------------------------------
-- Local routines to manipulate YYSTYPE objects
------------------------------------------------------------------

  -- ***********************************************************
  -- *                                                         *
  -- * Ada95_Parser.Set                                        *
  -- *                                                         *
  -- ***********************************************************
  procedure Set (Result       : out STRING_BUFFER;
		 Input_String : in STRING) is
    Temp_Result : STRING_BUFFER;
  begin -- Set
    if Input_String'LENGTH <= Maximum_String_Length then
      Temp_Result.Last := Input_String'LENGTH;
      Temp_Result.Text(1..Temp_Result.Last) := Input_String;
    else
      Temp_Result.Last := Maximum_String_Length;
      Temp_Result.Text(1..Temp_Result.Last) :=
        Input_String(Input_String'FIRST .. Input_String'FIRST - 1 +
                     Maximum_String_Length);
    end if;
    Result := Temp_Result;
  end Set;

------------------------------------------------------------------
-- Local routines to handle action rules in the grammar
------------------------------------------------------------------

------------------------------------------------------------------
-- Local routines for performing the parse
------------------------------------------------------------------

  -- ***********************************************************
  -- *                                                         *
  -- * Ada95_Parser.YYParse                                    *
  -- *                                                         *
  -- ***********************************************************
  --| PURPOSE:
  --| Perform the parse of a file.
  --| YYParse is inserted below

##%procedure_parse

------------------------------------------------------------------
-- Routines exported by the specification
------------------------------------------------------------------

  -- ***********************************************************
  -- *                                                         *
  -- * Ada95_Parser.Run                                        *
  -- *                                                         *
  -- ***********************************************************
  procedure Run (File_Name : in STRING) is
  --| PURPOSE:
  --| Cause the Parser to execute.  This is simply a call to YYParse
  --| with a update of statistics.
  begin -- Run
    A95.Reset (File_Name);
    Number_of_Errors := 0;
    begin
      YYParse;
    exception
      when others => null;
    end;
    A95.Close_Files;
  end Run;

  -- ***********************************************************
  -- *                                                         *
  -- * Ada95_Parser.Display_Results                            *
  -- *                                                         *
  -- ***********************************************************
  procedure Display_Results is
  --| PURPOSE:
  --| Display final results on all files processed so far.
  --| YOU HAVE TO FILL THIS IN
  begin -- Display_Results
    null;
  end Display_Results;

end A95P;


