-- **************************************
-- *                                    *
-- * Command Line Interface             * SPEC
-- *                                    *
-- **************************************
package Command_Line_Interface is
--| Purpose
--|   Command_Line_Interface is a package which provides
--| access to a Command Line.  It resembles the UNIX/C
--| command line interface, providing an argument
--| count and the arguments themselves.
--|
--| Initialization Exceptions (none)
--|
--| Notes
--|   Compiler limit on string length and dynamic memory.
--|
--| Modifications
--|  2/25/88  Richard Conn    Initial Version
--|  5/12/89  Richard Conn    Review and Upgrade
--|  4/11/90  Richard Conn    MIL-HDBK-1804 Annotations and
--|                           Meridian Ada Interface Added
--|  2/27/96  Richard Conn    Adapted from CS Parts for SCATKIT
   
  --***************************************
  --*                                     *
  --* Command_Line_Interface.Token_Count  *
  --*                                     *
  --***************************************
  function Token_Count return NATURAL;
  --| Purpose
  --|   Return the number (1 to N) of command line arguments.
  --| Token_Count is at least 1 because the name of the
  --| program or process is always Token_Value(0).
  --|
  --| Exceptions (none)
  --| Notes (none)

  --***************************************
  --*                                     *
  --* Command_Line_Interface.Token_Value  *
  --*                                     *
  --***************************************
  function Token_Value (Index : in NATURAL) return STRING;
  --| Purpose
  --|   Return the INDEXth (0 <= INDEX < Token_Count) command line
  --| argument.  Example: if ARGC = 1, ARGV(0) is the only
  --| valid argument string.  ARGV(0) is always the name of
  --| the program or process.
  --|
  --| Exceptions
  --|   INVALID_INDEX     raised if Index >= Token_Count
  --|
  --| Notes (none)

  INVALID_INDEX    : exception;
  UNEXPECTED_ERROR : exception;  -- raised anytime
   
end Command_Line_Interface;
