-- **********************************************
-- *                                            *
-- * Command_Line_Processor                     * SPEC
-- *                                            *
-- **********************************************
package Command_Line_Processor is
--| Purpose
--|   Command_Line_Processor is an abstract state machine
--| that allows the user to access a command line, which
--| may contain file references which are include files,
--| as a simple list of file names which can be accessed
--| via an interator and a Get function.  The command line
--| syntax is:
--|
--|      command input_file input_file ... input_file
--|
--| where any "input_file" may be prefixed by an "@"
--| to make it an include file.
--|
--| Modifications
--|   2/19/90  Rick Conn  Initial Design and Code
--|   2/27/96  Rick Conn  Adapted to SCATKIT

  --***********************************************
  --*                                             *
  --* Command_Line_Processor.Reset                *
  --*                                             *
  --***********************************************
  procedure Reset;
  --| Purpose
  --| Reset the iterator.  It is not necessary to
  --| call this routine the first time the processor
  --| is used.
  --|
  --| Exceptions (none)

  --***********************************************
  --*                                             *
  --* Command_Line_Processor.Number_of_File_Names *
  --*                                             *
  --***********************************************
  function Number_of_File_Names return NATURAL;
  --| Purpose
  --| Return the number of file names on the command line
  --| counting both the regular file names and the file
  --| names in the include files.
  --|
  --| Exceptions (none)

  --***********************************************
  --*                                             *
  --* Command_Line_Processor.Is_End_of_File_Names *
  --*                                             *
  --***********************************************
  function Is_End_of_File_Names return BOOLEAN;
  --| Purpose
  --| Return TRUE if no more file names are
  --| available.
  --|
  --| Exceptions (none)

  --***********************************************
  --*                                             *
  --* Command_Line_Processor.Next_File_Name       *
  --*                                             *
  --***********************************************
  function Next_File_Name return STRING;
  --| Purpose
  --| Return the name of the next file.  Advance the
  --| iterator.
  --|
  --| Exceptions
  --|   END_OF_FILE_LIST

  END_OF_FILE_LIST    : exception;

end Command_Line_Processor;
