-- *********************************************************
-- *                                                       *
-- *  Input_Text_File_Handler                              *  BODY
-- *                                                       *
-- *********************************************************
with Ada.Text_IO;
with Ada.Unchecked_Deallocation;
package body Input_Text_File_Handler is

  Current_FID : Ada.Text_IO.FILE_TYPE;
  Is_Open     : BOOLEAN := FALSE;

  type STRING_ACCESSOR is access STRING;
  No_File_Name      : STRING_ACCESSOR := new STRING'("<None>");
  Current_File_Name : STRING_ACCESSOR := No_File_Name;

  -- *****************************************************************
  -- *                                                               *
  -- *  Input_Text_File_Handler.Free                                 *
  -- *                                                               *
  -- *****************************************************************
  procedure Free is new Ada.Unchecked_Deallocation (STRING, STRING_ACCESSOR);

  -- *****************************************************************
  -- *                                                               *
  -- *  Input_Text_File_Handler.Open_File                            *
  -- *                                                               *
  -- *****************************************************************
  procedure Open_File (File_Name : in STRING) is
  begin -- Open_File
    Ada.Text_IO.Open (Current_FID, Ada.Text_IO.IN_FILE, File_Name);
    Is_Open := TRUE;
    if Current_File_Name /= No_File_Name then
      Free(Current_File_Name);
    end if;
    Current_File_Name := new STRING'(File_Name);
  exception
    when others => raise OPEN_ERROR;
  end Open_File;

  -- *****************************************************************
  -- *                                                               *
  -- *  Input_Text_File_Handler.Get_File_Name                        *
  -- *                                                               *
  -- *****************************************************************
  function Get_File_Name return STRING is
  begin -- Get_File_Name
    return Current_File_Name.all;
  end Get_File_Name;

  -- *****************************************************************
  -- *                                                               *
  -- *  Input_Text_File_Handler.Is_End_of_File                       *
  -- *                                                               *
  -- *****************************************************************
  function Is_End_of_File return BOOLEAN is
  begin -- Is_End_of_File
    if not Is_Open then
      raise FILE_NOT_OPEN;
    end if;
    return Ada.Text_IO.End_of_File(Current_FID);
  end Is_End_of_File;

  -- *****************************************************************
  -- *                                                               *
  -- *  Input_Text_File_Handler.Get_Line                             *
  -- *                                                               *
  -- *****************************************************************
  function Get_Line return STRING is
    Input_Line : STRING(1..Maximum_Line_Length);
    Input_Last : NATURAL;
  begin -- Get_Line
    if not Is_Open then
      raise FILE_NOT_OPEN;
    end if;
    if Is_End_of_File then
      raise END_OF_FILE;
    end if;
    Ada.Text_IO.Get_Line (Current_FID, Input_Line, Input_Last);
    return Input_Line (1..Input_Last);
  end Get_Line;

  -- *****************************************************************
  -- *                                                               *
  -- *  Input_Text_File_Handler.Close_File                           *
  -- *                                                               *
  -- *****************************************************************
  procedure Close_File is
  begin -- Close_File
    if Is_Open then
      Ada.Text_IO.Close(Current_FID);
      Is_Open := FALSE;
      Free(Current_File_Name);
      Current_File_Name := No_File_Name;
    end if;
  end Close_File;

end Input_Text_File_Handler;
