-- IC_TEST2.A
-- by Richard Conn
--
-- Test Information_Collector by using an array of integers
-- Add tests for Get_Last_Information_Packet and
--   Update_Last_Information_Packet
with Information_Collector;
with Console;
procedure IC_Test2 is
  type Integer_Array is array (INTEGER range <>) of INTEGER;

  Input_Array : constant Integer_Array :=
    (12, 2, 14, 100, -2, 35, 62, 62);

  Expected_Result_Array : constant Integer_Array :=
    (-1, 3, 13, 15, 36, 63, 63, 101);

  package Integer_Collector is new Information_Collector
    (Information_Packet => INTEGER,
     "<"                => Standard."<"); -- "<" for INTEGERs

  type RESULT is (INVALID_RESULT_AFTER_SORT, ADD_FAILURE, SORT_FAILURE,
                  SUCCESS);
  Result_Flag : RESULT := SUCCESS;

  ABORT_TEST : exception;

begin -- IC_Test2

  declare
    Value : INTEGER := 0;
  begin
    for I in Input_Array'range loop
      Integer_Collector.Add_Information_Packet (Input_Array(I));
      Value := Integer_Collector.Get_Last_Information_Packet;
      Value := Value + 1;
      Integer_Collector.Update_Last_Information_Packet (Value);
    end loop;
  exception
    when others =>
      Result_Flag := ADD_FAILURE;
      raise ABORT_TEST;
  end;

  begin
    Integer_Collector.Sort_Information_Packets;
  exception
    when others =>
      Result_Flag := SORT_FAILURE;
      raise ABORT_TEST;
  end;

  declare
    Rover : INTEGER := Expected_Result_Array'First;
    I     : INTEGER := 0;
  begin
    Integer_Collector.Reset_Iterator;
    while not Integer_Collector.Is_End_of_Packets loop
      Integer_Collector.Fetch_Next_Information_Packet
        (Next_Packet => I);
      if I /= Expected_Result_Array(Rover) then
        Result_Flag := INVALID_RESULT_AFTER_SORT;
        raise ABORT_TEST;
      end if;
      Rover := Rover + 1;
    end loop;
  end;

  Console.Put_Line ("Test Completed");
  Console.Put_Line ("  Result Flag: " & RESULT'IMAGE(Result_Flag));

exception -- IC_Test2
  when ABORT_TEST =>
    Console.Put_Line ("Test Aborted");
    Console.Put_Line ("  Result Flag: " & RESULT'IMAGE(Result_Flag));
  when others =>
    Console.Put_Line ("Test Aborted");
    Console.Put_Line ("  Unexpected exception");
end IC_Test2;

