-- ***********************************************************
-- *                                                         *
-- * LA95Test (Mainline Procedure)                           *
-- *                                                         *
-- ***********************************************************
with Command_Line_Processor;    -- SCATC DSK
with Console;                   -- SCATC DSK
with Output_Text_File_Handler;  -- SCATC DSK
with A95; use A95;              -- SCATC DSK Ada95 Lexical Analyzer
procedure LA95Test is -- Lexical Analyzer Testing Tool
--| PURPOSE:
--| LexTest is the main executive object for the system
--| of interacting objects.  It accesses the command line, moving
--| through it and invoking the methods of other objects as required.
--|
--| AUTHOR AND REVISION HISTORY:
--| Version  Date            Author          Notes
--|   1.0    22 Feb 95       Dean Runzel     Initial release
--|   1.1    24 Feb 95       Rick Conn       Minor fixes
--|   1.2     2 Apr 97       Rick Conn       Modify for SCATC Kit

  -- Counters
  CL_Item_Count : NATURAL := 0; -- Command Line Item Count

  -- Shorthand for the Command_Line_Processor and Output_Text_File_Handler
  package CLP renames Command_Line_Processor;
  -- package OF  renames Output_Text_File_Handler;

  Output_Text_File_Name : constant STRING := "la95test.out";

  package Output_File_Handler is new Output_Text_File_Handler;

  -- ***********************************************************
  -- *                                                         *
  -- * LexTest.Process_File                                    *
  -- *                                                         *
  -- ***********************************************************
  procedure Process_File (Arg : in STRING) is
  --| PURPOSE:
  --| Process Arg 

    This_Token : A95.Token;

  begin -- Process_File

    A95.Reset (Arg);
    Console.Put_Line ("Processing File " & Arg);
    Output_File_Handler.Put_Line("File Name: " & Arg);
    Output_File_Handler.Put_Line;
 
    loop
      
      This_Token := A95.Get_Token;
      Output_File_Handler.Put_Line ("This_Token = " &
        A95.Token'IMAGE(This_Token));
      exit when This_Token = END_OF_INPUT;
      Output_File_Handler.Put_Line ("  This_Token Value = " &
        A95.Get_Token_Value);

    end loop;

    A95.Close_Files;

  end Process_File;

  -- ***********************************************************
  -- *                                                         *
  -- * LA95Test (mainline)                                     *
  -- *                                                         *
  -- ***********************************************************
  
begin -- LA95Test

  -- Show that the tool has started
  Console.Put_Line ("Ada95 Lexical Analyzer Test Program");
  Console.Put_Line ("Output in " & Output_Text_File_Name);

  -- Initialize Command Line Processor and check for empty line
  CLP.Reset;
  CL_Item_Count := CLP.Number_of_File_Names;
  
  if CL_Item_Count < 1 then

    Console.Put_Line
    ("Syntax: LA95Test ada_source_file_names*");

  else

    -- Open output file
    Output_File_Handler.Create_File(Output_Text_File_Name);

    -- Process for command line options and files
    Scan_Command_Line:
    while not CLP.Is_End_of_File_Names loop
      Process_File (CLP.Next_File_Name);
    end loop Scan_Command_Line;

  end if;

  -- Close output file
  Output_File_Handler.Close_File;

end LA95Test;
